package  jiniexamples.chapter10;

import net.jini.space.JavaSpace;
import net.jini.lookup.ServiceDiscoveryManager;
import net.jini.core.lookup.ServiceTemplate;
import net.jini.core.lookup.ServiceItem;
import net.jini.lease.LeaseRenewalManager;
import net.jini.core.lease.Lease;
import net.jini.core.transaction.server.TransactionManager;
import net.jini.core.transaction.Transaction;
import net.jini.core.transaction.TransactionFactory;
import net.jini.core.entry.Entry;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.MouseAdapter;
import java.awt.event.WindowAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JPanel;

import java.util.Vector;
import java.rmi.RMISecurityManager;


public class RoomBrowser extends JPanel  {
  static ServiceDiscoveryManager sdm;
  static String username = null;
  static JavaSpace javaspace;
  static TransactionManager transManager;
  static LeaseRenewalManager leaseManager;

  protected final JList roomList;

  protected long duration = 60 * 60 * 1000; // 1 Hour...

  RoomBrowser(JavaSpace jspace, TransactionManager txm,
              LeaseRenewalManager lrm)  {

    javaspace = jspace; // == Get handle of filestore
    transManager = txm;
    leaseManager = lrm;

    //SET UP THE TOOLBAR.
    JPanel toolBar = new JPanel();


    Vector rooms = getRooms();

    roomList = new JList(rooms);
    roomList.addMouseListener( new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        if (e.getClickCount() == 2) {
          int index = roomList.locationToIndex(e.getPoint());

          roomList.setSelectedIndex(index);

          String name = (String) roomList.getSelectedValue();
          RoomChatWindow roompanel =
                 new  RoomChatWindow(name,username,javaspace,
                                     transManager,leaseManager);

          JFrame roomframe = new JFrame("Room :" + name);
          roomframe.setContentPane(roompanel);
          roomframe.pack();
          roomframe.addWindowListener(new WindowAdapter() {
             public void windowClosing(WindowEvent e) {
                JFrame frame = (JFrame) e.getSource();
                RoomChatWindow roomwindow =
                         (RoomChatWindow) frame.getContentPane();
                leaveRoom(roomwindow.getRoom());
                }
          });
          roomframe.setVisible(true);

        }
      }
    });

    JScrollPane roomPane = new JScrollPane(roomList);

    setLayout(new BorderLayout());
    setPreferredSize(new Dimension(400, 300));
    add(roomPane, BorderLayout.CENTER);
  }


  public void leaveRoom(String roomname) {

    try {
        Transaction.Created trans =
               TransactionFactory.create(transManager,duration);
        leaseManager.renewUntil(trans.lease,Lease.FOREVER, null);
        Transaction txn= trans.transaction;
        Room theroom =
           (Room) javaspace.takeIfExists(new Room(roomname,null),
                                         txn, duration);
        if (theroom != null) {
            theroom.members.remove(username);
            Lease ls = javaspace.write(theroom,txn, duration);
            leaseManager.renewUntil(ls, Lease.FOREVER, null);
        }
        System.out.println("Written  " + theroom);
        txn.commit();
        System.out.println("Change committed  " + theroom);
        leaseManager.remove(trans.lease);

    } catch (Exception ex) {
        System.err.println("Error ammending room members: " +
                           ex.getMessage());
    }
  }

  public Vector getRooms(){
    try {
         Room room =null;
         Vector returnval = new Vector();
         Transaction.Created trans =
           TransactionFactory.create(transManager,Lease.FOREVER);

         leaseManager.renewUntil(trans.lease,Lease.FOREVER,null);
         Entry template = javaspace.snapshot(new Room());
         while ((room = (Room)javaspace.take(template,
                             trans.transaction, 200) ) != null){
                returnval.add(room.name);
         }
         trans.transaction.abort();
         leaseManager.remove(trans.lease);
         return returnval;
    } catch (Exception ex) {
         System.err.println("Could not fetch message " + ex);
         return null;
    }
  }


  public static Object getService(ServiceDiscoveryManager sdm,
                                  Class classname){

    Class[] type = new Class[] {classname };
    ServiceTemplate template = 
                             new ServiceTemplate(null,type,null);
    try {
       ServiceItem serviceitem =
                      sdm.lookup(template, null, Long.MAX_VALUE);
        // Use the Service if has been found.
       if ( serviceitem == null ) {
           System.out.println("Can't find service");
           return null;
       } else {
           return serviceitem.service;
       }
    } catch (Exception e) {
          System.out.println("Exception finding service :" + e);
          return null;
    }
  }

  public static void main (String[] args)  {

    if (args.length != 1) {
       System.out.println("Usage: <username>");
       System.exit(1);
    }
    try {
       System.setSecurityManager (new RMISecurityManager ());

       sdm = new ServiceDiscoveryManager(null,null);
       username = args[0];

       JavaSpace jspace =
                  (JavaSpace) getService(sdm, JavaSpace.class);
       TransactionManager txmanager =
             (TransactionManager) getService(sdm,
                                      TransactionManager.class);
       if  ((jspace != null) && ( txmanager != null)){
           RoomBrowser rooms =
                          new RoomBrowser(jspace, txmanager,
                                  sdm.getLeaseRenewalManager() );
           JFrame browserframe =
                     new JFrame("Space Chat Rooms :" + username);
           browserframe.addWindowListener(new WindowAdapter() {
             public void windowClosing(WindowEvent e) {
                 sdm.terminate();
                 System.exit(0);
             }
           });
           browserframe.setContentPane(rooms);
           browserframe.pack();
           browserframe.setVisible(true);
        }
    } catch (Exception e) {
        System.out.println ("main()exception:" + e);
    }
  }
}
