package jiniexamples.chapter10;

import net.jini.core.entry.Entry;
import net.jini.core.lease.Lease;
import net.jini.core.transaction.server.TransactionManager;
import net.jini.core.transaction.Transaction;
import net.jini.core.transaction.TransactionFactory;
import net.jini.space.JavaSpace;
import net.jini.lease.LeaseRenewalManager;

import javax.swing.JTextField;
import javax.swing.JTextArea;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JPanel;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.util.Vector;


public class RoomChatWindow extends JPanel  {

  protected JavaSpace javaspace;
  protected TransactionManager transManager;
  protected LeaseRenewalManager leaseManager;
  protected String thisuser;
  protected String roomname;

  protected  JList userList;
  protected  JTextArea textWindow;

  protected JTextField message;
  protected long duration = 60 * 60 * 1000;

  public RoomChatWindow(String room, String user,
                        JavaSpace jspace, TransactionManager txm,
                        LeaseRenewalManager lrm)  {
    javaspace = jspace;
    transManager = txm;
    leaseManager = lrm;
    thisuser = user;
    roomname = room;

    JPanel chatBar = new JPanel();

    JTextField message = new JTextField();

    message.setPreferredSize(new Dimension(300, 25));

    message. addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
            JTextField messagefield = (JTextField)e.getSource();
            String messagetext = messagefield.getText();
            sendMessage(messagetext);
            messagefield.setText("");
        }
    });

    //  Register for message based additions to the JavaSpace.
    try {
         javaspace.notify(new RoomSpeak(roomname,null,null),
                          null,new MessageHandler(jspace,this),
                          duration, null);
    } catch (Exception ex) {
          System.err.println("Error setting up notification: " +
                             ex.getMessage());
    }
    chatBar.add(message);

    Vector userlist = enterRoom(thisuser);
    userList = new JList(userlist);
    //  Register for room based additions to the JavaSpace.
        try {
              javaspace.notify(new Room(roomname,null),
                               null,new RoomHandler(jspace,this),
                               duration, null);
        } catch (Exception ex) {
            System.err.println("Error setting up notification: " 
                                + ex.getMessage());
        }

        JScrollPane userPane = new JScrollPane(userList);
        textWindow = new JTextArea();

        this.buildHistory();

        JScrollPane textPane = new JScrollPane(textWindow);

        //LAY OUT THE VARIOUS COMPONENTS.

        setLayout(new BorderLayout());
        setPreferredSize(new Dimension(400, 300));
        add(chatBar, BorderLayout.NORTH);
        add(userPane, BorderLayout.WEST);
        add(textPane, BorderLayout.CENTER);
  }

  public void sendMessage(String message ){
    try {
       Transaction.Created trans =
                TransactionFactory.create(transManager,duration);
       leaseManager.renewUntil(trans.lease, duration, null);
       RoomSpeak lastmessage =
                   (RoomSpeak) javaspace.takeIfExists(
                               new RoomSpeak(roomname,null,null),
                               trans.transaction,duration);
        if (lastmessage != null) {
            Lease l = javaspace.write(new RoomHistory(roomname,
                                        lastmessage.speaker,
                                        lastmessage.message),
                                        trans.transaction, 
                                        duration);
            leaseManager.renewUntil(l, Lease.FOREVER, null);
        }
        RoomSpeak utterance = 
                        new RoomSpeak(roomname,thisuser,message);
        Lease l =javaspace.write(utterance, trans.transaction, 
                                 duration);
        leaseManager.renewUntil(l, Lease.FOREVER, null);

        trans.transaction.commit();
        leaseManager.remove(trans.lease);
    } catch (Exception ex) {
          System.err.println("Error writing event: " + ex);
    }
  }


  public void buildHistory(){
     RoomHistory messobj =null;

     try {
       Transaction.Created trans =
           TransactionFactory.create(transManager,Lease.FOREVER);
       leaseManager.renewUntil(trans.lease, Lease.FOREVER, null);

       Entry template = 
         javaspace.snapshot(new RoomHistory(roomname,null,null));

       while ((messobj = (RoomHistory)javaspace.take(template,
                                        trans.transaction, 200) )
               != null){
                appendMessage(messobj);
       }
       trans.transaction.abort();
       leaseManager.remove(trans.lease);

       RoomSpeak lastmessage =
           (RoomSpeak) javaspace.readIfExists(
                               new RoomSpeak(roomname,null,null),
                               null,200);
       if (lastmessage != null) {
               appendMessage(lastmessage );
       }
    } catch (Exception ex) {
         System.err.println("Could not fetch message " + ex);
    }
  }

  public Vector enterRoom(String user){
      try {
          Transaction.Created trans =
               TransactionFactory.create(transManager,duration);
          leaseManager.renewUntil(trans.lease, duration, null);
          Room theroom = 
           (Room) javaspace.takeIfExists(new Room(roomname,null),
                                         null, duration);
          if (theroom != null) {
              theroom.members.add(user);
              Lease l = 
                      javaspace.write(theroom,trans.transaction, 
                                      duration);
              leaseManager.renewUntil(l, Lease.FOREVER, null);
              trans.transaction.commit();
              leaseManager.remove(trans.lease);
              return theroom.members;
          }

     } catch (Exception ex) {
          System.err.println("Error ammending room members: " +
                              ex.getMessage());
     }
     return null;
  }

  public String getRoom() {
     return roomname;
  }

  public void appendMessage(Message messobj ){
    textWindow.append(messobj.speaker+">"+messobj.message +"\n");
  }

  public void updateUserList(Vector  list ){
      userList.setListData(list);
  }
}

