package jiniexamples.chapter11;

import net.jini.core.lookup.ServiceID;
import net.jini.lookup.ServiceIDListener;
import net.jini.core.entry.Entry;
import net.jini.lookup.entry.Name;
import net.jini.discovery.LookupDiscoveryManager;
import net.jini.core.event.RemoteEventListener;

import net.jini.lookup.ServiceDiscoveryManager;

import net.jini.discovery.LookupDiscovery;
import net.jini.discovery.LookupDiscoveryService;
import net.jini.discovery.DiscoveryListener;
import net.jini.discovery.DiscoveryEvent;
import net.jini.discovery.LookupDiscoveryManager;
import net.jini.discovery.LookupDiscoveryRegistration;
import net.jini.discovery.LookupUnmarshalException;

import net.jini.core.lookup.ServiceRegistrar;
import net.jini.core.lookup.ServiceItem;
import net.jini.core.lookup.ServiceRegistration;
import net.jini.core.lookup.ServiceTemplate;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.lease.Lease;

import net.jini.lease.LeaseRenewalService;
import net.jini.lease.LeaseRenewalSet;
import net.jini.lease.LeaseRenewalManager;

import java.rmi.RemoteException;
import java.rmi.Remote;
import java.rmi.MarshalledObject;
import java.rmi.RMISecurityManager;

import java.rmi.activation.Activatable;
import java.rmi.activation.ActivationDesc;
import java.rmi.activation.ActivationGroup;
import java.rmi.activation.ActivationGroupID;
import java.rmi.activation.ActivationGroupDesc;
import java.rmi.activation.ActivationException;

import java.util.Properties;


public class ActivationServiceWrapper  {



  public ActivationServiceWrapper(String[] args) {

    Entry[] attributes =  new Entry[] { new Name(args[0])};;

    try{

      if (System.getSecurityManager() == null) {
           System.setSecurityManager( new RMISecurityManager());
      }
      String[] allgroups = new String[] { "" };

      ServiceDiscoveryManager sdm =
                          new ServiceDiscoveryManager(null,null);
      setupActivationGroup (args[1]);

      // SERVICE SETUP STUFF
      RemoteFileStore fileservice =
              (RemoteFileStore) createActivatableObject(
                  "jiniexamples.chapter11.ActivatableFileStore",
                   args[2],
                   null );

      RemoteResourceServiceProxy serviceproxy =
                   new RemoteResourceServiceProxy(fileservice);

      ServiceItem item =
                 new ServiceItem(null, serviceproxy, attributes);

      // LEASE SETUP STUFF
      RemoteEventListener leaseHandler =
              (RemoteEventListener) createActivatableObject(
                      "jiniexamples.chapter11.RenewLeaseHandler",
                       args[2],
                       null );

      LeaseRenewalSet leaseRenewalSet =
                            getLeaseService( sdm, leaseHandler );
      if (leaseRenewalSet == null) {
           System.out.println(" Leasing Service not found");
                System.exit(1);
      }

      // DISCOVERY STUFF
      // Pass the Service Item and the LeaseReneval
      //Service to the Discovery Handler
      Object[] discoveryInfo = {item,leaseRenewalSet};
      MarshalledObject discoveryData = null;
      try {
          discoveryData = new MarshalledObject(discoveryInfo);
      } catch(java.io.IOException e) {
          e.printStackTrace();
      }

      RemoteEventListener discoveryStub =
              (RemoteEventListener)createActivatableObject(
                    "jiniexamples.chapter11.DiscoveryChange",
                     args[2],
                     discoveryData );

      setupDiscoveryService(sdm,leaseRenewalSet,item,
                            discoveryStub);

       sdm.terminate();  System.exit(0);

     } catch (Exception ex) {
       ex.printStackTrace();
     }
  }

  public LeaseRenewalSet getLeaseService(
                              ServiceDiscoveryManager sdm,
                              RemoteEventListener leaseHandler ){
                              	
      LeaseRenewalService leaseService =
            (LeaseRenewalService) getService(sdm,
                                     LeaseRenewalService.class);
      if (leaseService == null) {
          System.out.println(" NEW Lease service null");
          return null;
      }
      try {
          LeaseRenewalSet leaseRenewalSet =
                       leaseService.createLeaseRenewalSet(1000);
          leaseRenewalSet.setExpirationWarningListener(
                                                    leaseHandler,
                                                    300,
                                                    null);
          return leaseRenewalSet;
      } catch(RemoteException e) {
          e.printStackTrace();
          return null;
      }
   }

   public void setupDiscoveryService(ServiceDiscoveryManager sdm,
                             LeaseRenewalSet leaseRenewalSet,
                             ServiceItem item,
                             RemoteEventListener discoveryStub){

      LookupDiscoveryService discoveryService =
        (LookupDiscoveryService) getService(sdm,
                                   LookupDiscoveryService.class);
      if (discoveryService == null) {
            return;
      }
      LookupDiscoveryRegistration registration = null;
      try {
          registration =
          discoveryService.register(LookupDiscovery.ALL_GROUPS,
                                null,
                                discoveryStub,
                                null,
                                Lease.FOREVER);
      } catch(RemoteException e) {
          e.printStackTrace();
      }
       // Add a lease for the  discovery service to the Lease Set
      try {
          leaseRenewalSet.renewFor(registration.getLease(),
                                     Lease.FOREVER);
      } catch(RemoteException e) {
          e.printStackTrace();
      }
   }

   public static Object getService(ServiceDiscoveryManager sdm,
                                   Class classname){
      Class[] type = new Class[] {classname};
      ServiceTemplate template = 
                             new ServiceTemplate(null,type,null);
      try {
         ServiceItem serviceitem =
                     sdm.lookup(template, null, Long.MAX_VALUE);
         // Use the Service if has been found.
         if ( serviceitem == null ) {
           System.out.println("Can't find service");
           return null;
         } else {
           return serviceitem.service;
         }
      } catch (Exception e) {
         System.out.println("Exception finding service :" + e);
         return null;
      }
  }

  protected void setupActivationGroup (String policyFile){
      try {
        Properties props = new Properties();
        props.put("java.security.policy", policyFile);
        ActivationGroupDesc group =
                            new ActivationGroupDesc(props, null);
        // Register the group and get the ID.
        ActivationGroupID activationID  =
                ActivationGroup.getSystem().registerGroup(group);
                // Now create the group.
        ActivationGroup.createGroup(activationID, group, 0);
      } catch (RemoteException ex) {
          System.err.println("Error registering"+
                             "ActivationGroupDesc:"+
                              ex.getMessage());
          System.exit(1);
          return ; // not reached
      } catch (ActivationException ex) {
          System.err.println("Problem with activation: " +
                              ex.getMessage());
           System.exit(1);
           return ; // not reached
      }
   }

  protected  Object createActivatableObject(String classname,
                      String codebase, MarshalledObject data ) {
      try {
        // Create a descriptor for a new activation group
        ActivationDesc desc =
                 new ActivationDesc(classname,codebase, data);

        return Activatable.register(desc);
      } catch (RemoteException ex) {
        System.err.println(
             "Error registering the ActivationGroupDesc: " +
              ex.getMessage());
        ex.printStackTrace();
        System.exit(1);
        return null;
      } catch (ActivationException ex) {
        System.err.println("Problem with activation: " +
                               ex.getMessage());
        ex.printStackTrace();
        System.exit(1);
        return null;
      }
  }

 public static void main(String[] args) {
    if (args.length != 3) {
        System.out.println("Usage :ActivationServiceWrapper"+
                           "FilestoreName PolicyFile Codebase");
        System.exit(0);
    } else {
        new ActivationServiceWrapper(args);
    }
  }
}
