package jiniexamples.chapter11;

import java.rmi.activation.Activatable;
import java.rmi.activation.ActivationID;
import java.rmi.MarshalledObject;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.lookup.ServiceID;
import net.jini.core.lookup.ServiceItem;
import net.jini.core.lookup.ServiceRegistrar;
import net.jini.core.lookup.ServiceRegistration;
import net.jini.lease.LeaseRenewalSet;
import net.jini.discovery.RemoteDiscoveryEvent;
import java.rmi.RemoteException;
import  net.jini.discovery.LookupUnmarshalException;

import jiniexamples.chapter4.RemoteResourceStore;

public class DiscoveryHandler extends Activatable
                                implements RemoteEventListener {

  protected LeaseRenewalSet leaseRenewalSet;
  protected RemoteResourceStore serviceproxy;
  protected ServiceItem item = null;
  protected ServiceID servid = null;
  protected long leaseDuration = 24*60*60*1000; // 1 Day



  public DiscoveryHandler(ActivationID id,MarshalledObject data)
                                         throws RemoteException {
    super(id, 0);
    Object[] objs = null;
    try {
        objs = (Object []) data.get();
    } catch(Exception e) {
        System.out.println ("Error getting data" + e);;
    }
    item = (ServiceItem) objs[0];
    leaseRenewalSet= (LeaseRenewalSet) objs[1];
  }



  public void notify(RemoteEvent evnt) {
    RemoteDiscoveryEvent discevnt = (RemoteDiscoveryEvent) evnt;

    if (! discevnt.isDiscarded() && item != null) {
        if (item.serviceID != null) { // Has a service ID
           servid=item.serviceID;
        } else if (servid != null) { // Use the first service ID
           item.serviceID = servid;
        }

        ServiceRegistrar[] registrars = null;
        try {
            registrars = discevnt.getRegistrars();
        } catch(LookupUnmarshalException e) {
            System.out.println ("Unmarshall error" + e);
           return;
        }

        for (int n = 0; n < registrars.length; n++) {
            ServiceRegistrar registrar = registrars[n];
            ServiceRegistration reg = null;
            try {
                reg = registrar.register(item, leaseDuration);
                leaseRenewalSet.renewFor(reg.getLease(), 
                                         leaseDuration);
                servid = reg.getServiceID();
            } catch(RemoteException e) {
                 System.err.println("Register exception: " + 
                                     e.toString());
            }
        }
    }
      // Make it inactive again now you have register the service
    try {
            Activatable.inactive(this.getID());
    } catch (Exception e) {
           System.out.println ("error making inactive " + e);
    }
  }
}

