package jiniexamples.chapter11;

import net.jini.core.lookup.ServiceID;
import net.jini.lookup.ServiceIDListener;
import net.jini.core.entry.Entry;
import net.jini.lookup.entry.Name;
import net.jini.lookup.JoinManager;

import net.jini.discovery.LookupDiscoveryManager;

import java.rmi.RemoteException;
import java.rmi.Remote;
import java.rmi.MarshalledObject;
import java.rmi.RMISecurityManager;

import java.rmi.activation.Activatable;
import java.rmi.activation.ActivationDesc;
import java.rmi.activation.ActivationGroup;
import java.rmi.activation.ActivationGroupID;
import java.rmi.activation.ActivationGroupDesc;
import java.rmi.activation.ActivationException;

import java.util.Properties;



public class ServiceWrapper implements ServiceIDListener {

  protected JoinManager join = null;
  protected ServiceID servID;

   // The code used by Join Manager
  public void serviceIDNotify(ServiceID id) {
    servID = id;
    System.out.println("New version The service ID is " + id);

    // Pause for a while and then shut down the starter
    System.out.println("Shut down the service Wrapper");
    System.exit(0);
  }


  public ServiceWrapper(String[] args) {

    Entry[] attributes =  new Entry[] { new Name(args[0])};;

    // Create the instance of the service; the JoinManager will
    // register it and renew its leases with a lookup service

    try{

      if (System.getSecurityManager() == null) {
            System.setSecurityManager( new RMISecurityManager());
      }

      System.out.println( "Create...Activatable Object");
      setupActivationGroup (args[1]); 
      ActivatableFileStore fileservice =
              (ActivatableFileStore) createActivatableObject(
                  "jiniexamples.chapter11.ActivatableFileStore",
                                   args[2],null );

      System.out.println( "Create Service Proxy...");
      RemoteResourceServiceProxy serviceproxy =
                new RemoteResourceServiceProxy(fileservice);


      String[] allgroups = new String[] { "" };

      LookupDiscoveryManager discoverymanager =
              new LookupDiscoveryManager(allgroups, null, null);
      join = new JoinManager(serviceproxy, attributes,
                             this, discoverymanager, null);

    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }


  protected  Object createActivatableObject(String classname,
                       String codebase, MarshalledObject data ) {
    try {
      // Create a descriptor for a new activation group
      ActivationDesc desc =
                 new ActivationDesc(classname,codebase, data);

      return Activatable.register(desc);

    } catch (RemoteException ex) {
      System.err.println(
             "Error registering the ActivationGroupDesc: " +
              ex.getMessage());
      ex.printStackTrace();
      System.exit(1);
      return null;
    } catch (ActivationException ex) {
      System.err.println("Problem with activation: " +
                               ex.getMessage());
      ex.printStackTrace();
      System.exit(1);
      return null; // not reached
    }
  }


  protected void setupActivationGroup (String policyFile) {
     try {
        Properties props = new Properties();
        props.put("java.security.policy", policyFile);
        ActivationGroupDesc group =
                            new ActivationGroupDesc(props, null);
        // Register the group and get the ID.
        ActivationGroupID activationID  =
                ActivationGroup.getSystem().registerGroup(group);
                // Now create the group.
        ActivationGroup.createGroup(activationID, group, 0);

        } catch (RemoteException ex) {
           System.err.println( "Error with ActivationGroupDesc: "
                               + ex.getMessage());
           ex.printStackTrace();
           System.exit(1);
           return ;
        } catch (ActivationException ex) {
           System.err.println("Problem with activation: " +
                               ex.getMessage());
           ex.printStackTrace();
           System.exit(1);
           return ; // not reached
        }
   }

  public static void main(String[] args) {
    if (args.length != 3) {
         System.out.println("Usage :ServiceWrapper FilestoreName" +
                            " PolicyFile Codebase");
         System.exit(0);
    } else {
         new ServiceWrapper(args);
    }

  }
}
