package jiniexamples.chapter12;

import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JPanel;
import javax.swing.JComboBox;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.MouseAdapter;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.ActionEvent;

import java.rmi.RemoteException;
import java.net.URL;

import jiniexamples.chapter4.RemoteResourceStore;
import jiniexamples.chapter4.RemoteResource;


public class RemoteFileBrowser extends JPanel {

    protected RemoteResourceStore fileStore;
    protected final JList dataList;
    protected JComboBox dirListing;
    protected boolean dirselectActive =true;


    public RemoteFileBrowser(RemoteResourceStore remoteFiles) {

        fileStore = remoteFiles; // == Get handle of filestore

        //SET UP THE TOOLBAR.
        JPanel toolBar = new JPanel();
        addButtons(toolBar);

        // SET UP THE FILE DATA LIST
        dataList = new JList();

        // file renderer
        dataList.setCellRenderer(new FileCellRenderer());
        //specialised mousehandler for doubleclicks
        dataList.addMouseListener( new MouseAdapter() {
        public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                  int index = 
                          dataList.locationToIndex(e.getPoint());
                  dataList.setSelectedIndex(index);
                  RemoteResource selectedFile =
                    (RemoteResource) dataList.getSelectedValue();
                  if (selectedFile.isDirectory()) {
                             changeToDirectory(selectedFile);
                        }
                 }
              }
        });

        JScrollPane listPane = new JScrollPane(dataList);

        //LAY OUT THE VARIOUS COMPONENTS.
        setLayout(new BorderLayout());
        setPreferredSize(new Dimension(400, 300));
        add(toolBar, BorderLayout.NORTH);
        add(listPane, BorderLayout.CENTER);

        // SETUP THE DIRECTORY.
        try {
             changeToDirectory(fileStore.getCurrentDir());
        } catch (RemoteException ex) {
             System.out.println(" Remote Error :.." +ex);
        }
    }

    protected void addButtons(JPanel controlPanel) {

        // Combo box showing the name of the current director
        dirListing = new JComboBox();
        dirListing.addActionListener(new ActionListener() {
           public void actionPerformed(ActionEvent e) {
                JComboBox cb = (JComboBox)e.getSource();
                RemoteResource dir = 
                          (RemoteResource) cb.getSelectedItem();
                if (dir != null) {
                        if (dirselectActive){
                                changeToDirectory(dir);
                        }

                  }
                }
        });

        dirListing.setRenderer(new DirCellRenderer());
        dirListing.setPreferredSize(new Dimension(300, 23));

        JPanel dirPanel = new JPanel();
        dirPanel.add(dirListing);

        ImageIcon upIcon = null;
        try {
           upIcon = 
                new ImageIcon(
                       new URL ("http://roddent/images/up.gif"));
        } catch (Exception e) {
           upIcon = null;
        }
        //first button
        JButton upbutton  =null;
        if (upIcon != null) {
              upbutton  = new JButton(upIcon);
        } else {
             upbutton  = new JButton("UP");
        }
        upbutton.setToolTipText("Change UP a directory");
        upbutton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
              try {
                  fileStore.changeUpdir();
                  changeToDirectory(fileStore.getCurrentDir());
              } catch (RemoteException ex) {
                  System.out.println(" Remote Error :.." +ex);
              }
            }
        });
        controlPanel.add(dirPanel, BorderLayout.WEST);
        controlPanel.add(upbutton, BorderLayout.EAST);
    }

    protected void changeToDirectory(RemoteResource dir){
        RemoteResource dirHierarchy = dir;
        try {
            fileStore.setCurrentDir(dir);
            // Set  the COMBOBOX showing hierarchy
            dirselectActive = false;

            dirListing.removeAllItems();
            while (dirHierarchy != null){
                dirListing.addItem(dirHierarchy);
                dirHierarchy= fileStore.getParent(dirHierarchy);
            }
            dirselectActive = true;
            RemoteResource[] listing = fileStore.listResources();
            dataList.setListData(listing);
        } catch (RemoteException ex) {
            System.out.println(" Remote Error :.." +ex);
        }
     }

}

