package jiniexamples.chapter12;


import net.jini.core.lookup.ServiceID;
import net.jini.core.discovery.LookupLocator;
import net.jini.core.entry.Entry;
import net.jini.lookup.JoinManager;
import net.jini.lookup.ServiceIDListener;
import net.jini.discovery.LookupDiscoveryManager;

import net.jini.lookup.ui.MainUI;
import net.jini.lookup.ui.factory.JFrameFactory;
import net.jini.lookup.entry.UIDescriptor;
import net.jini.lookup.ui.attribute.UIFactoryTypes;


import java.io.IOException;
import java.rmi.RMISecurityManager;
import java.rmi.MarshalledObject;

import jiniexamples.chapter4.FileStoreService;
import jiniexamples.chapter4.RemoteResourceServiceProxy;

import java.util.Set;
import java.util.HashSet;


public class ServiceUIWrapper implements ServiceIDListener {

  protected JoinManager join = null;

  public void serviceIDNotify(ServiceID serviceID)  {
      System.out.println("The Service ID is " + serviceID);
  }


  public ServiceUIWrapper() {
  }

  public ServiceUIWrapper(Object serviceproxy) 
                                            throws IOException {

      Set typeNames = new HashSet();
      typeNames.add(JFrameFactory.TYPE_NAME);

      // The attributes set
      Set attribs = new HashSet();
      attribs.add(new UIFactoryTypes(typeNames));

      // Create a marshalled object of the factory
      MarshalledObject factory = null;
      try {
          factory = 
            new MarshalledObject(new RemoteFileBrowserFactory());
      } catch(Exception ex) {
          System.out.println("Exception Marshelling Factory" 
                              +ex);
          System.exit(2);
      }

      // Create the description..
      UIDescriptor uiDesc = new UIDescriptor(MainUI.ROLE,
                                RemoteFileBrowserFactory.TOOLKIT,
                                attribs,
                                factory);

      Entry[] attributes = {uiDesc};

     // Add the Service to the Jini community .
      String[] allgroups = new String[] { "" };
      LookupDiscoveryManager discoverymanager = 
               new LookupDiscoveryManager(allgroups, null, null);
      join = new JoinManager(serviceproxy, attributes, this, 
                             discoverymanager, null);

  }

  public static void main(String[] args) {
  
      if (System.getSecurityManager() == null) {
          System.setSecurityManager( new RMISecurityManager());
      }

      try{
          FileStoreService fileservice = 
                       (FileStoreService) new FileStoreService();
          RemoteResourceServiceProxy serviceproxy = 
                     new RemoteResourceServiceProxy(fileservice);

          ServiceUIWrapper wrapper = 
                             new ServiceUIWrapper(serviceproxy);
      } catch (Exception ex) {
          ex.printStackTrace();
      }
  }
}