package jiniexamples.chapter12;

import java.rmi.RMISecurityManager;
import net.jini.core.lookup.ServiceTemplate;
import net.jini.core.lookup.ServiceItem;
import net.jini.core.entry.Entry;

import net.jini.lookup.ServiceDiscoveryManager;
import net.jini.lookup.ui.MainUI;
import net.jini.lookup.ui.factory.JFrameFactory;
import net.jini.lookup.entry.UIDescriptor;
import net.jini.lookup.ui.attribute.UIFactoryTypes;

import javax.swing.JFrame;
import java.util.Set;
import java.util.Iterator;
import java.lang.ClassLoader;

import net.jini.discovery.LookupDiscoveryManager;

import jiniexamples.chapter4.RemoteResourceStore;


public class TestRemoteBrowser {

  static ClassLoader  loader;

  public TestRemoteBrowser() {
        // Set it up so we know the loader for this
        // class
        loader = this.getClass().getClassLoader();
        }

  public static void main(String argv[]) {

       // Set a security manager.
       if (System.getSecurityManager() == null) {
             System.setSecurityManager(new RMISecurityManager());
       }

       try {

           String[] allgroups = new String[] { "" };
           TestRemoteBrowser browser = new TestRemoteBrowser();
           LookupDiscoveryManager discoverymanager =
               new LookupDiscoveryManager(allgroups, null, null);
           ServiceDiscoveryManager sdm =
             new ServiceDiscoveryManager(discoverymanager,null);

           // Set up the template to search by class and UIDescriptor
           Class[] classes = 
                         new Class[] {RemoteResourceStore.class};

           UIDescriptor desc = new UIDescriptor(MainUI.ROLE,
                                          JFrameFactory.TOOLKIT,
                                          null, null);
           Entry [] entries = {desc};

           ServiceTemplate template =
                     new ServiceTemplate(null, classes, entries);

           // Block until a matching service is found
           System.out.println("Looking for Service...");
           ServiceItem serviceitem = 
                     sdm.lookup(template, null, Long.MAX_VALUE);
           // Use the Service if one has been found.
           if ( serviceitem == null ) {
                System.out.println("Can't find service");
           } else {
                System.out.println("Service Found ID =" 
                                    + serviceitem.serviceID);
                Entry[] attributes = serviceitem.attributeSets;
                for (int i = 0; i < attributes.length; i++) {
                   Entry attr = attributes[i];
                   if (attr instanceof UIDescriptor) {
                        showInterface(serviceitem, 
                                     (UIDescriptor) attr );
                    }
                }
            }
        } catch (Exception ex)   {
             System.out.println (" main() exception: " + ex);
             ex.printStackTrace();
        }
    }

    private static void showInterface(ServiceItem item, 
                                      UIDescriptor desc) {
        Set attributes = desc.attributes;
        Iterator descAtts = attributes.iterator();
        while (descAtts.hasNext()) {
            //  find the Factory for the User Interfaces
            Object obj = descAtts.next();
            if (obj instanceof UIFactoryTypes) {
                UIFactoryTypes objtype = (UIFactoryTypes) obj;

                if (objtype.isAssignableTo(JFrameFactory.class)){
                    JFrameFactory factory = null;
                    try {
                      factory = 
                       (JFrameFactory) desc.getUIFactory(loader);
                    } catch(Exception e) {
                      e.printStackTrace();
                      continue;
                    }

                    // CREATE THE INTERFACE AND SHOW IT
                    JFrame frame = factory.getJFrame(item);
                    if (frame == null) {
                        System.out.println("Null Frame");
                    } else {
                        frame.setVisible(true);
                    }
                }
            }
        }
    }
}
