// A simple client to exercise the CurrencyConvertService

package jiniexamples.chapter2;


import net.jini.discovery.DiscoveryListener;
import net.jini.discovery.DiscoveryEvent;
import net.jini.discovery.LookupDiscoveryManager;
import net.jini.core.lookup.ServiceRegistrar;
import net.jini.core.lookup.ServiceTemplate;
import java.io.IOException;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;

public class CurrencyConvertClient implements Runnable {
    protected ServiceTemplate template;
    protected LookupDiscoveryManager disco;
    
    // An inner class to implement DiscoveryListener
    class Listener implements DiscoveryListener {
      
       public void discovered(DiscoveryEvent ev) {
            ServiceRegistrar[] newregs = ev.getRegistrars();
            if (newregs != null) {
               for (int i=0 ; i<newregs.length ; i++) {
                    // Find the service 
                       CurrencyConvertServiceInterface servobj = 
                       (CurrencyConvertServiceInterface)
                                      lookForService(newregs[i]);
                    // Use the Service
                       if (servobj!=null) {
                        System.out.println("The value of 500" +
                                  "dollars in UK Pounds is  " + 
                                  servobj.convert(500,"UK" ));
                        }
                 }
            }
        } 
       

        public void discarded(DiscoveryEvent ev) {
        }
    }
    
    public CurrencyConvertClient() throws IOException {
        Class[] types = {CurrencyConvertServiceInterface.class};
        
        template = new ServiceTemplate(null, types, null);
        
        
        // Set a security manager.
        if (System.getSecurityManager() == null) {
            System.setSecurityManager(
                    new RMISecurityManager());
        }
        
        // Create a discovery Listener 
        
        DiscoveryListener myListener = new Listener();
         
        // Search for the "public" group, which by
        // convention is named by the empty string.
        
        
        
        disco = new LookupDiscoveryManager(new String[] { "" },
                                null, myListener);
       
    
    }
    
    // Once we've found a new lookup service, search
    // for proxies that implement 
    // CurrencyConvertServiceInterface.
    protected Object lookForService(ServiceRegistrar 
                                    lusvc) {
        Object o = null;
        
        try {
        	        System.out.println("Looking for service");
            o = lusvc.lookup(template);
        } catch (RemoteException ex) {
            System.err.println("Error doing lookup: " +
                               ex.getMessage());
            return null;
        }
        
        if (o == null) {
            System.err.println("No matching service.");
            return null;
        }
        
        System.out.println("Got a matching service.");     
        return o;
    }
    
    
    
    // This thread does nothing--it simply keeps the
    // VM from exiting while we do discovery.
    public void run() {
        while (true) {
            try {
                Thread.sleep(1000000);
            } catch (InterruptedException ex) {
            }
        }
    }
    

    // Create a CurrencyConvertClient and start its thread.
    public static void main(String args[]) {
        try {
           System.out.println("Client is Started ");
           CurrencyConvertClient hwc = new CurrencyConvertClient();
            new Thread(hwc).start();
        } catch (IOException ex) {
            System.out.println("Couldn't create client: " +
                               ex.getMessage());
        }
    }
}
