// This is the first iteration of a simple currency
// converter service--it publishes a proxy that returns
// a a new currency value when asked by clients.


package jiniexamples.chapter2;

import net.jini.discovery.DiscoveryListener;
import net.jini.discovery.DiscoveryEvent;
import net.jini.discovery.LookupDiscoveryManager;
import net.jini.core.lookup.ServiceItem;
import net.jini.core.lookup.ServiceRegistrar;
import net.jini.core.lookup.ServiceRegistration;
import java.util.HashMap;
import java.util.Hashtable;
import java.io.IOException;
import java.io.Serializable;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;

// This is the proxy object that will be downloaded
// by clients.  It's serializable and implements
// our well-known CurrencyConvertServiceInterface.
class CurrencyConvertServiceProxy implements Serializable,
    CurrencyConvertServiceInterface {

    Hashtable exchangerate = new Hashtable();

    public CurrencyConvertServiceProxy () {
    }

// This initializes the converter to allow conversions for 
// known countries.
    public void initRates(){
       exchangerate.put("UK",new Float(0.72 ));
       exchangerate.put("Sweden",new Float(200));
    }
    
// add a new currency exchange rate
    public void addRates(String country, float rate){
       exchangerate.put(country,new Float(rate ));
    }
    

// The convert routine converts known values else it returns  
// the same value back.
    public float convert(float amount, String country){
    	
    	Float xchrate = (Float)exchangerate.get(country);
    	if (xchrate != null)
        	return amount*(xchrate.floatValue()) ;
          else
          	return -1; // -1.0  to indicate no conversion
    }
}
// CurrencyConvertService is the "wrapper" class that
// handles publishing the service item.
public class CurrencyConvertService implements Runnable {
    // 10 minute leases
    protected final int LEASE_TIME = 10 * 60 * 1000; 
    protected HashMap registrations = new HashMap();
    protected ServiceItem item;
    protected LookupDiscoveryManager discoverymanager;
    
    // Inner class to listen for discovery events
    class Listener implements DiscoveryListener {
 
        // Called when we find a new lookup service.
        public void discovered(DiscoveryEvent ev) {
            System.out.println("discovered a lookup " +
                               "service!");
            ServiceRegistrar[] newregs = ev.getRegistrars();
            for (int i=0 ; i<newregs.length ; i++) {
                if (!registrations.containsKey(newregs[i])){
                    registerWithLookup(newregs[i]);
                }
            }
        }
        
        // Called ONLY when we explicitly discard a
        // lookup service, not "automatically" when a
        // lookup service goes down.  Once discovered,
        // there is NO ongoing communication with a
        // lookup service.
        public void discarded(DiscoveryEvent ev) {
            ServiceRegistrar[] deadregs = 
                ev.getRegistrars();
            for (int i=0 ; i<deadregs.length ; i++) {
                registrations.remove(deadregs[i]);
            }
        }
    }
    
        public CurrencyConvertService() throws IOException {
        	
        
        // Create an instance of the service proxy
        CurrencyConvertServiceProxy proxyobj= 
                  createCurrencyProxy();
        
        
        // Create a service item to be added to the lookup 
        // service        
        item = new ServiceItem(null, proxyobj, null);
        
        // Set a security manager.
        if (System.getSecurityManager() == null) {
            System.setSecurityManager(
                    new RMISecurityManager());
        }
        
        // Create a discovery Listener 
        
        DiscoveryListener myListener = new Listener();
         
        // Search for the "public" group, which by
        // convention is named by the empty string.
        
        
        
        discoverymanager = 
                 new LookupDiscoveryManager(new String[] { "" },
                                            null, myListener);
        
    }
    
    
    protected CurrencyConvertServiceProxy createCurrencyProxy() {
        CurrencyConvertServiceProxy proxyobj= 
                 new CurrencyConvertServiceProxy();
        proxyobj. initRates();
        return proxyobj;
    }
    
    
    // This work involves remote calls, and may take a
    // while to complete.  Thus, since it's called from
    // discovered(), it will prevent us from responding
    // in a timely fashion to new discovery events.  An
    // improvement would be to spin off a separate short-
    // lived thread to do the work.
    protected synchronized void registerWithLookup(
                                  ServiceRegistrar  registrar) {
        ServiceRegistration registration = null;

        try {
            registration = registrar.register(item,
                                              LEASE_TIME);
        } catch (RemoteException ex) {
            System.out.println("Couldn't register: " +
                               ex.getMessage());
            return;
        }
        // If this is our first registration, use the
        // service ID returned to us.  Ideally, we should
        // save this ID so that it can be used after
        // restarts of the service.
        if (item.serviceID == null) {
            item.serviceID = registration.getServiceID();
            System.out.println("Set serviceID to " +
                               item.serviceID);
        }
        
        registrations.put(registrar, registration);
    }
    
    // This thread does nothing but sleep, but it
    // makes sure the VM doesn't exit.
    public void run() {
        while (true) {
            try {
                Thread.sleep(1000000);
            } catch (InterruptedException ex) {
            }
        }
    }
    
    // Create a new CurrencyConvertService and start
    // its thread.
    public static void main(String args[]) {
        try {
            System.out.println("Started .... ");
            CurrencyConvertService hws = 
                new CurrencyConvertService();
            new Thread(hws).start();
        } catch (IOException ex) {
            System.out.println("Couldn't create service: " +
                               ex.getMessage());
        }
    }
}

