// Extend the client to renew its event registration
// leases.
package jiniexamples.chapter3;

import net.jini.core.lookup.ServiceRegistrar;
import net.jini.core.event.EventRegistration;
import net.jini.core.lease.Lease;
import net.jini.lease.LeaseListener;
import net.jini.lease.LeaseRenewalEvent;
import net.jini.lease.LeaseRenewalManager;
import java.io.IOException;
import java.rmi.RemoteException;

public class CurrencyConvertClientWithLeases 
    extends CurrencyConvertClientWithEvents 
    implements LeaseListener {
        
    protected LeaseRenewalManager lrm;
       
    // Notifier called by when the Lease Renewal manager
    // cannot renew the lease. 
    public void notify(LeaseRenewalEvent e) {
    	System.out.println("Couldn't renew Lease: " +  
                               e.getLease());
    }  
    
    public CurrencyConvertClientWithLeases()
        throws RemoteException, IOException {       
         lrm = new LeaseRenewalManager();
    }

    // When we register for events, add the event's
    // registration lease to the leases managed by the
    // LeaseRenewalManager.
    protected void registerForEvents(ServiceRegistrar lu) 
        throws RemoteException {
        EventRegistration evreg;
    
        evreg = lu.notify(template, 
                    ServiceRegistrar.TRANSITION_NOMATCH_MATCH,
                    eventCatcher, null, LEASE_TIME);            
        lrm.renewFor(evreg.getLease(),Lease.FOREVER,this );
    }
    
    // Start the service.
    public static void main(String args[]) {
        try {
            CurrencyConvertClientWithLeases hwc = 
                new CurrencyConvertClientWithLeases();
        } catch (IOException ex) {
            System.out.println("Couldn't create client: " +
                               ex.getMessage());
        }
    }
}
    
