// A version of CurrencyConvertService that creates a "back-end"
// remote object for the proxy to communicate with.

package jiniexamples.chapter3;

import jiniexamples.chapter2.CurrencyConvertServiceInterface;

import net.jini.core.lookup.ServiceItem;
import java.util.Hashtable;
import java.io.IOException;
import java.io.Serializable;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

// CurrencyConvertServiceBackend is the "wrapper" application
// that creates a remote object and publishes a proxy that
// can refer to it.


public class CurrencyConvertServiceBackend
    extends CurrencyConvertServiceWithLeases {

    // Backend is the RMI server object that receives
    // remote method invocations from the proxy.  It simply
    // cycles through a set of strings each time it is
    // called.

   //

    static class Backend extends UnicastRemoteObject
        implements BackendProtocol {


	Hashtable exchangerate = new Hashtable();


        // Nothing for the constructor to do but let the
        // superclass constructor run.
        Backend() throws RemoteException {
	}

	// This initializes the converter to allow conversions
	// for known countries.
    	public void initRates(){
       		exchangerate.put("UK",new Float(0.72 ));
       		exchangerate.put("Sweden",new Float(200));
    	}

	// add a new currency exchange rate
    	public void addRates(String country, float rate){
       		exchangerate.put(country,new Float(rate ));
    	}

        public float doConversion(float amount, String country)
                                          throws RemoteException{
            System.out.println("Back-end: doing the conversion");
            Float xchrate = (Float)exchangerate.get(country);
            if (xchrate != null)
                return amount*(xchrate.floatValue()) ;
            else
               return amount;
        }
    }

    // Constructor doesn't have to do anything
    // except let the superclass constructor run.
    public CurrencyConvertServiceBackend() throws IOException {
    }


    // Since we're using a different proxy, we have to
    // reimplement createCurrencyProxy().  This new version 
    // creates a remote server object that will receive method
    // invocations from the proxy, and creates a
    // CurrencyConvertServiceProxy2 object that refers to it.
    protected CurrencyConvertServiceInterface createCurrencyProxy() {
        try {
            Backend backend = new Backend();
            backend.initRates();
            return new CurrencyConvertServiceProxy2(backend);
        } catch (RemoteException ex) {
            System.err.println("Error creating backend: " +
                               ex.getMessage());
            ex.printStackTrace();
            System.exit(1);
            return null;        // NOT REACHED
        }
    }

    // Main creates the wrapper
    public static void main(String args[]) {
        try {
            CurrencyConvertServiceBackend hws =
                new CurrencyConvertServiceBackend();
            new Thread(hws).start();
        } catch (IOException ex) {
            System.out.println("Couldn't create service: " +
                               ex.getMessage());
        }
    }
}

