// Extend CurrencyConvertService to renew its service
// registration leases.
package jiniexamples.chapter3;

import jiniexamples.chapter2.CurrencyConvertService;

import net.jini.core.lookup.ServiceRegistrar;
import net.jini.core.lookup.ServiceRegistration;
import net.jini.core.lease.Lease;
import net.jini.lease.LeaseListener;
import net.jini.lease.LeaseRenewalEvent;
import net.jini.lease.LeaseRenewalManager;
import java.io.IOException;
import java.rmi.RemoteException;

public class CurrencyConvertServiceWithLeases 
    extends CurrencyConvertService 
    implements LeaseListener {
    
    protected LeaseRenewalManager lrm;
    
    // Notifier called by when the Lease Renewal manager
    // cannot renew the lease. 
    public void notify(LeaseRenewalEvent e) {
    	System.out.println("Couldn't renew Lease: " +  
                               e.getLease());
    }      
    
    public CurrencyConvertServiceWithLeases() 
        throws IOException {
        lrm = new LeaseRenewalManager();
    }

    protected synchronized void registerWithLookup(
                                   ServiceRegistrar registrar) {
        ServiceRegistration registration = null;
        try {
            registration = registrar.register(item,
                                              LEASE_TIME);
        } catch (RemoteException ex) {
            System.out.println("Couldn't register: " +
                               ex.getMessage());
            return;
        }        
        if (item.serviceID == null) {
            item.serviceID = registration.getServiceID();
            System.out.println("Set serviceID to " +
                               item.serviceID);
        }       
        registrations.put(registrar, registration);
        // Hand the lease over to the LeaseRenewalManager
        lrm.renewFor(registration.getLease(),
                     Lease.FOREVER,this );
    }
        
    // Create the service 
    public static void main(String args[]) {
        try {
            CurrencyConvertServiceWithLeases hws = 
                new CurrencyConvertServiceWithLeases();
            new Thread(hws).start();
        } catch (IOException ex) {
            System.out.println("Couldn't create service: " +
                               ex.getMessage());
        }
    }
}
