// This is the remote filestore service and it implements the
// RemoteFileStore interface. This implementation provides the
// services to be implemented by looking at a local filestore.  
  
package jiniexamples.chapter4;

import java.io.File;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

public class FileStoreService extends UnicastRemoteObject 
                  implements RemoteFileStore  {

	
  protected RemoteResource currentDir  = new RemoteResource();
    
  public  FileStoreService() throws RemoteException{	
        
       File f = new  File( File.separator);
       
       currentDir.setContents (f);
       currentDir.setName(f.getName());
       currentDir.setDirectory(f.isDirectory());	
  }

    
  public RemoteResource getCurrentDir() throws RemoteException{
       return currentDir;
  }
  
  public void setCurrentDir(RemoteResource dir) 
                                          throws RemoteException{
       currentDir =  dir;
  }
    
  
  public RemoteResource getParentFile(RemoteResource dir) 
                                          throws RemoteException{
       
       File file = (File)dir.getContents();
       File parent = file.getParentFile();
       
       if (parent != null) {
          RemoteResource parentDir = new RemoteResource();
          parentDir.setContents (parent);
          parentDir.setName(parent.getName());
          parentDir.setDirectory(parent.isDirectory());
          return parentDir;
       } else {
          return null;
       }
   }
   
   
  public void changeUpdir() throws RemoteException{
       
       RemoteResource parent = this.getParentFile(currentDir); 
       if (parent != null)
          currentDir =parent;
   }
   

  public RemoteResource[] listFiles() throws RemoteException{
       
       File file = (File)currentDir.getContents();
       File [] filelist = file.listFiles();
       
       if (filelist != null) {
          RemoteResource[] returnlist= 
                             new RemoteResource[filelist.length];
          for (int i = 0; i< filelist.length; i++){
           returnlist[i] = new RemoteResource();
           returnlist[i].setContents (filelist[i]);
           returnlist[i].setName(filelist[i].getName());
           returnlist[i].setDirectory(filelist[i].isDirectory());
          }
          return returnlist;
       } else {
          return null;
       }
   }
 }
