package jiniexamples.chapter4;

import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JPanel;
import javax.swing.JComboBox;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.MouseAdapter;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.ActionEvent;

import net.jini.core.entry.Entry;
import net.jini.lookup.ServiceDiscoveryManager;
import net.jini.core.lookup.ServiceTemplate;
import net.jini.core.lookup.ServiceItem;

import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;


public class RemoteFileBrowser extends JPanel {
	
  protected RemoteResourceStore fileStore;
  protected final JList dataList;
  protected JComboBox dirListing;
  protected boolean dirselectActive =true;
    
  static ServiceDiscoveryManager sdm;
    
  public RemoteFileBrowser(RemoteResourceStore remoteFiles) {
       
       fileStore = remoteFiles; // == Get handle of filestore

        //SET UP THE TOOLBAR.
       JPanel toolBar = new JPanel();
       addButtons(toolBar);
         
       // SET UP THE FILE DATA LIST
       dataList = new JList(); 
       	
       // file renderer 
       dataList.setCellRenderer(new FileCellRenderer());
       //specialised mousehandler for doubleclicks	
       dataList.addMouseListener( new MouseAdapter() {
          public void mouseClicked(MouseEvent e) {
           if (e.getClickCount() == 2) {
              int index = dataList.locationToIndex(e.getPoint());
              
              dataList.setSelectedIndex(index);
              RemoteResource selectedFile =
                    (RemoteResource) dataList.getSelectedValue();
              if (selectedFile.isDirectory()) {
                  changeToDirectory(selectedFile);
              }	
            }	
          }
         });
       
       JScrollPane listPane = new JScrollPane(dataList);
       
        //LAY OUT THE VARIOUS COMPONENTS.
       setLayout(new BorderLayout());
       setPreferredSize(new Dimension(400, 300));
       add(toolBar, BorderLayout.NORTH);
       add(listPane, BorderLayout.CENTER);

        // SETUP THE DIRECTORY.
       try { 
           changeToDirectory(fileStore.getCurrentDir());
       } catch (RemoteException ex) {
           System.out.println(" Remote Error :.." +ex);
       }       
  }

  protected void addButtons(JPanel controlPanel) {
	
       // Combo box showing the name of the current director	
       dirListing = new JComboBox();	   
       dirListing.addActionListener(new ActionListener() {
           public void actionPerformed(ActionEvent e) {
               JComboBox cb = (JComboBox)e.getSource();
                RemoteResource dir = 
                           (RemoteResource) cb.getSelectedItem();
                if (dir != null) {
                  if (dirselectActive){
                        changeToDirectory(dir);
                  }
                }
          }
         });
	
       dirListing.setRenderer(new DirCellRenderer());
             dirListing.setPreferredSize(new Dimension(300, 23));
	       	
       JPanel dirPanel = new JPanel();
       //dirPanel.setPreferredSize(new Dimension(300, 50));
       dirPanel.add(dirListing);
                     
        //first button
       JButton upbutton  = 
            new JButton(new ImageIcon("c:/files/images/up.gif"));
       upbutton.setToolTipText("Change UP a directory");
       upbutton.addActionListener(new ActionListener() {
           public void actionPerformed(ActionEvent e) {
              try { 
                  fileStore.changeUpdir();
                  changeToDirectory(fileStore.getCurrentDir());
              } catch (RemoteException ex) {
                  System.out.println(" Remote Error :.." +ex);
             }
           }
         });
                   
       controlPanel.add(dirPanel, BorderLayout.WEST);  
       controlPanel.add(upbutton, BorderLayout.EAST);       
  }
    
  protected void changeToDirectory(RemoteResource dir){       
          
       RemoteResource dirHierarchy = dir;         

       try { 
           fileStore.setCurrentDir(dir);
           dirselectActive = false;
          	
           dirListing.removeAllItems();	
           while (dirHierarchy != null){
             dirListing.addItem(dirHierarchy);
              dirHierarchy= fileStore.getParent(dirHierarchy); 
           }
           dirselectActive = true;	
                       
           RemoteResource[] listing = fileStore.listResources();
           dataList.setListData(listing); 
            
       } catch (RemoteException ex) {
           System.out.println(" Remote Error :.." +ex);
       }
  }
        
  public static void main (String[] args) {
        // Set a security manager.
       if (System.getSecurityManager() == null) {
             System.setSecurityManager(new RMISecurityManager());
       }
       try {
          sdm = new ServiceDiscoveryManager(null,null);
     
          // Set up the template to search by class
          Class[] name = new Class[] {RemoteResourceStore.class}; 
          ServiceTemplate template = 
                             new ServiceTemplate(null,name,null);

           // Block until a matching service is found
          System.out.println("Looking for Service...");
          ServiceItem serviceitem = 
                      sdm.lookup(template, null, Long.MAX_VALUE);
           // Use the Service if one has been found.
          if ( serviceitem == null ) {
              System.out.println("Can't find service");
          } else {       
              System.out.println("Service Found with ID..."+ 
                                           serviceitem.serviceID);
              RemoteResourceStore servobj = 
                       (RemoteResourceStore) serviceitem.service;         
                   // Create the browser and place it in a frame
              RemoteFileBrowser contentPane = 
                                  new RemoteFileBrowser(servobj);
              JFrame browserframe = 
                     new JFrame("Simple RemoteResource Browser");
              browserframe.addWindowListener(new WindowAdapter(){
                  public void windowClosing(WindowEvent e) {
                             sdm.terminate();
                      System.exit(0);
                  }
                });
          
              browserframe.setContentPane(contentPane);
              browserframe.pack();
              browserframe.setVisible(true);          
          }
       } catch (Exception e)   {
          System.out.println ("client: exception: " + e);
       }
  } 
}

