/* This proxy creates an adaptor proxy object that maps between
   the interface provided to the Jini framework and the RMI 
   interface used to communicate with the remote service */

package jiniexamples.chapter4;

import java.io.Serializable;
import java.rmi.Remote;
import java.rmi.RemoteException;

public class RemoteResourceServiceProxy  
                                 implements RemoteResourceStore,
                                            Serializable {
             	
    protected RemoteFileStore backend;

    public RemoteResourceServiceProxy() { //Not normally called
    }
    
    public RemoteResourceServiceProxy(RemoteFileStore backend){
       this.backend = backend;
    }
    
    // Note each of the methods map between the methods in 
    // the RemoteResourceStore interface and the 
    // RemoteFileStore interface. 
    

     // GETCURRENTDIR
    public RemoteResource getCurrentDir() 
                                       throws RemoteException {
       return backend.getCurrentDir();
    }
     // SETCURRENTDIR
    public void setCurrentDir(RemoteResource dir) 
                                       throws RemoteException{
       backend.setCurrentDir(dir);     
    }  
    // GETPARENT
    public RemoteResource getParent(RemoteResource dir) 
                                       throws RemoteException {
       return backend.getParentFile(dir);
    }
    // CHANGEUPDIR
    public void changeUpdir() throws RemoteException {
       backend.changeUpdir();  
    }   
    // LISTRESOURCES
    public RemoteResource[] listResources() 
                                       throws RemoteException{
       return backend.listFiles();
    }      
}
