package jiniexamples.chapter6;

import jiniexamples.chapter4.RemoteResourceStore;
import jiniexamples.chapter4.RemoteFileBrowser;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JPanel;

import java.awt.BorderLayout;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import net.jini.lookup.ServiceDiscoveryManager;
import net.jini.lookup.LookupCache;
import net.jini.lookup.ServiceDiscoveryListener;
import net.jini.lookup.ServiceDiscoveryEvent;
import net.jini.core.lookup.ServiceTemplate;
import net.jini.core.lookup.ServiceItem;

import java.util.Hashtable;
import java.rmi.RMISecurityManager;

public class FileServiceMonitor extends JPanel 
                            implements ServiceDiscoveryListener {
	
    protected  JList dataList;
    protected Hashtable knownServices = new Hashtable();
    
  // === ServiceDiscovery Listener Interface=====
  public void serviceAdded(ServiceDiscoveryEvent ev) { 
       ServiceItem item = ev.getPostEventServiceItem();
       knownServices.put(item.serviceID, item);
       dataList.setListData(knownServices.values().toArray());
  }

  public void serviceRemoved(ServiceDiscoveryEvent ev) {
       ServiceItem item = ev.getPreEventServiceItem();
       knownServices.remove(item.serviceID);
       dataList.setListData(knownServices.values().toArray());
  }
   
  public void serviceChanged(ServiceDiscoveryEvent ev) {
      System.out.println("SERVICE" +
                         ev.getPostEventServiceItem().serviceID +
                         "Changed");
  }
  // == End of ServiceDiscovery Interface ========

  public FileServiceMonitor(ServiceTemplate template, 
                       ServiceDiscoveryManager serviceManager ) {
       
       try {
            LookupCache serviceCache = 
                   serviceManager.createLookupCache(template,
                                                    null, this);
       } catch (Exception e) {
            System.out.println (" Monitor exception:" + e);
       }
            
      // Label .
       JLabel toolLabel= new JLabel("Available File Stores");
      // Service List
       dataList = new JList(); 
       dataList.setCellRenderer(new ServiceCellRenderer());
      
      //specialised mousehandler for doubleclicks	
       dataList.addMouseListener( new MouseAdapter() {
          public void mouseClicked(MouseEvent e) {
             if (e.getClickCount() == 2) {
                int index = 
                         dataList.locationToIndex(e.getPoint());
                dataList.setSelectedIndex(index);
                ServiceItem item =
                       (ServiceItem) dataList.getSelectedValue();
                RemoteFileBrowser remotebrowser = 
                       new RemoteFileBrowser(
                            (RemoteResourceStore) item.service );

                JFrame browserframe = new JFrame("Browser");
                browserframe.setContentPane(remotebrowser);
                browserframe.pack();
                browserframe.setVisible(true); 
             }	 
          }
       });
       JScrollPane listPane = new JScrollPane(dataList);
       setLayout(new BorderLayout());
       add(toolLabel, BorderLayout.NORTH);
       add(listPane, BorderLayout.CENTER);
  }

  public static void main (String[] args) {
       try {
           System.setSecurityManager (new RMISecurityManager ());
           ServiceDiscoveryManager sdm = 
                          new ServiceDiscoveryManager(null,null);
           Class[] name = new Class[]{RemoteResourceStore.class};

           ServiceTemplate template = 
                             new ServiceTemplate(null,name,null);
           FileServiceMonitor contentPane = 
                            new FileServiceMonitor(template,sdm);

           JFrame monitorframe = new JFrame("Filestore Monitor");
            
           monitorframe.addWindowListener(new WindowAdapter() {
                  public void windowClosing(WindowEvent e) {
                     System.exit(0);
                  }
           });
           monitorframe.setContentPane(contentPane);
           monitorframe.pack();
           monitorframe.setVisible(true);          
       } catch (Exception e)  {
           System.out.println ("client: main() exception: " + e);
       }
  }
}

