package jiniexamples.chapter7;

import java.util.Vector;
import net.jini.core.lease.Lease;
import com.sun.jini.lease.landlord.LeaseManager;
import com.sun.jini.lease.landlord.LeasedResource;
import com.sun.jini.lease.landlord.LeaseDurationPolicy;
import com.sun.jini.lease.landlord.Landlord;
import com.sun.jini.lease.landlord.LandlordLease;
import com.sun.jini.lease.landlord.LeasePolicy;
import java.util.Map;

public class LeaseManagerImpl implements LeaseManager {

  protected static long DEFAULT_TIME = 15000L; // 15 Secs
  protected Vector resources = new Vector();  
  protected LeaseDurationPolicy lease_policy; // Policy Manager.
    
  // =========== Lease Manager Interface =============
  public void register(LeasedResource rsrc, long duration) {
       resources.add(rsrc);
  }

  public void renewed(LeasedResource rsrc, long duration, 
                      long expired) {	
  }
  /// =========== Lease Manager Interface =============
  
  public LeaseManagerImpl() {
  }

  public LeaseManagerImpl(Landlord landlord) {
    	
       // Create a policy object
       lease_policy = new LeaseDurationPolicy(Lease.FOREVER, 
                                    DEFAULT_TIME, landlord,this,
                                    new LandlordLease.Factory());
       //Start a thread to check the leases 
       new CheckLeases().start();
  }
    
  public Map cancelAll(Object[] cookies) {
       for (int n =0; n <cookies.length; n++) {
           cancel(cookies[n]);
       }
       return null;
  }
    
  public void cancel(Object cookie) {
       for (int n =0; n <resources.size(); n++) {
           ManagedLeasedResource rsrc = 
                  (ManagedLeasedResource) resources.elementAt(n);
           if (rsrc.getCookie().equals(cookie)) {
                  resources.removeElementAt(n);	
           }
       }
  }

  public Vector managedResources() {
       return resources;
  }

  public LeasePolicy getPolicy() {
       return lease_policy;
  }

  public ManagedLeasedResource getResource(Object cookie) {
       for (int n = resources.size(); --n >= 0; ) {
           ManagedLeasedResource rsrc = 
                 (ManagedLeasedResource) resources.elementAt(n);
           if (rsrc.getCookie().equals(cookie)) {
               return rsrc;
           }
       }
       return null;
  }

  class CheckLeases extends Thread {
       public void run() {
          while (true) {
              try {
                  Thread.sleep(DEFAULT_TIME) ;
              } catch (InterruptedException e) {
              }
              for (int n =0; n <resources.size(); n++) {
                 ManagedLeasedResource rsrc = 
                  (ManagedLeasedResource) resources.elementAt(n);
                 if (!lease_policy.ensureCurrent(rsrc)) {
                     System.out.println("Lease expired for  " 
                                         + rsrc.getCookie()) ;
                     resources.removeElementAt(n);

                 }
              }
          }
       }
  }
}
