package jiniexamples.chapter8;

import java.util.Hashtable;
import java.util.Enumeration;

import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;

import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.Box;
import javax.swing.BoxLayout;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.Color;

import net.jini.lookup.ServiceDiscoveryManager;
import net.jini.lookup.LookupCache;
import net.jini.lookup.ServiceDiscoveryListener;
import net.jini.lookup.ServiceDiscoveryEvent;
import net.jini.lookup.entry.Name;
import net.jini.core.lookup.ServiceTemplate;
import net.jini.core.lookup.ServiceItem;
import net.jini.core.lookup.ServiceID;
import net.jini.core.lease.Lease;
import net.jini.lease.LeaseRenewalManager;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.EventRegistration;



public class FileActivityMonitor extends JPanel implements 
					ServiceDiscoveryListener{

  protected Box serviceList;
  protected ServiceDiscoveryManager serviceManager;
  protected LookupCache serviceCache;
  protected RemoteEventListener eventListener = null;
  protected static Hashtable knownRegistrations = new Hashtable();
  protected Hashtable actvityIndicators = new Hashtable();
 
    
    
  protected String serviceName (ServiceItem item) {
       for (int i=0 ; i<item.attributeSets.length ; i++) {
         if (item.attributeSets[i] instanceof Name) {
             return  ((Name)  item.attributeSets[i]).name;
             }
          }
       return "Unamed Service";
    }
    
     
    
    // === ServiceDiscovery Listener Interface=====
  public void serviceAdded(ServiceDiscoveryEvent ev) { 
  
        ServiceItem item = ev.getPostEventServiceItem();
        
        JButton service_label = new JButton(serviceName(item));
        service_label.setBackground(Color.white );
        
        serviceList.add(service_label);
        serviceList.validate();

        LeaseRenewalManager lrm = 
                         serviceManager.getLeaseRenewalManager();
        RemoteStoreWithEvents serviceProxy = 
                            (RemoteStoreWithEvents) item.service;
        
         // Register for the events and hand the lease over to 
         // the lease renewal manager.
        try {
             EventRegistration er = 
                    serviceProxy.register(FileUseEvent.Type,null, 
                                    eventListener,Lease.FOREVER);
             lrm.renewUntil(er.getLease(), Lease.FOREVER, null);
        
             // Remember details of registration
             knownRegistrations.put(item.serviceID, er);
             actvityIndicators.put(item.serviceID, service_label);
        } catch ( RemoteException ex) {
             System.out.println( " Error registering for events" 
                                 + ex);
        }
    }

  public void serviceRemoved(ServiceDiscoveryEvent ev) {
    	
    System.out.println("SERVICE REMOVED");
    ServiceItem item = ev.getPreEventServiceItem();
    ServiceID servID= item.serviceID;

    knownRegistrations.remove(servID);
        
    serviceList.remove((Component)actvityIndicators.get(servID));
    serviceList.validate();
    actvityIndicators.remove(servID);
  }
   
  public void serviceChanged(ServiceDiscoveryEvent ev) {
        System.out.println("SERVICE" 
                        + ev.getPostEventServiceItem().serviceID 
                        +"Changed");
  }

    // == End of ServiceDiscovery Interface ========
    
  public FileActivityMonitor(ServiceTemplate template, 
                               ServiceDiscoveryManager sdm )  {
       
      try {
           serviceManager = sdm;
           serviceCache = 
           serviceManager.createLookupCache(template, 
                                            null, this);
      } catch (Exception e) {
           System.out.println ("Create Service exception: " + e);
      }

     // Set up Event Handling
      try {
           eventListener = new FileUseHandler(actvityIndicators);
      } catch (RemoteException e)  {
          System.out.println ("Problem File Use Handler:" + e);
      }
            
      // Label 
      JLabel toolLabel= new JLabel("Activity on File Stores");
      // Service List
      serviceList = new Box(BoxLayout.Y_AXIS);
      
      
      JScrollPane listPane = new JScrollPane(serviceList);

      setLayout(new BorderLayout());
      add(toolLabel, BorderLayout.NORTH);
      add(listPane, BorderLayout.CENTER);
  }

   public static void main (String[] args) {
      if (System.getSecurityManager() == null) {
          System.setSecurityManager( new RMISecurityManager());
      }
      try {
          System.setSecurityManager (new RMISecurityManager ());
          ServiceDiscoveryManager sdm = 
                          new ServiceDiscoveryManager(null,null);
          Class[] name = 
                      new Class[] {RemoteStoreWithEvents.class}; 

          ServiceTemplate template = 
                            new ServiceTemplate(null,name,null);
          FileActivityMonitor contentPane = 
                           new FileActivityMonitor(template,sdm);
            

          JFrame monitorframe = new JFrame("File Store Monitor");
          monitorframe.addWindowListener(new WindowAdapter() {
                 public void windowClosing(WindowEvent e) {
           
                       // Cancel Leases
                     try {
                        for (Enumeration el = 
                             knownRegistrations.elements() ; 
                             el.hasMoreElements() ;) {
                             
                           EventRegistration er = 
                             (EventRegistration)el.nextElement();
                                      er.getLease().cancel();
                        }
 
                     } catch (Exception ex)  {
                         System.out.println ("Problem cancelling" 
                                             + ex);
                     }
                     System.exit(0);
                  }
          });
        	
          monitorframe.setContentPane(contentPane);
          monitorframe.setSize(200,300);
          monitorframe.setVisible(true);          
      } catch (Exception e)  {
          System.out.println ("client:main() exception: " + e);
      }
   }
}
