package jiniexamples.chapter8;

import jiniexamples.chapter4.RemoteResource;

import java.util.List;
import java.util.ArrayList;
import java.io.File;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.rmi.MarshalledObject;
import java.rmi.server.UnicastRemoteObject;

import com.sun.jini.lease.landlord.LandlordLease;
import net.jini.core.lookup.ServiceID;
import net.jini.lookup.ServiceIDListener;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.EventRegistration;
import net.jini.core.event.UnknownEventException;


// =============================================
// The cookie class for  leases.  This class
// identifies leases by their event type
// identifiers 

class EventCookie implements Serializable {
    long eventType;
    
    EventCookie(long eventType) {
        this.eventType = eventType;
    }
    
    public boolean equals(Object other) {
        if (!(other instanceof EventCookie))
            return false;
        
        EventCookie cookie = (EventCookie) other;
        
        return cookie.eventType == eventType;
    }
}

//====================================================


// ============================================================

public class FileStoreService extends UnicastRemoteObject 
                              implements RemoteFileStore, 
                                         ServiceIDListener  {
	
  protected RemoteResource rootDir  = new RemoteResource();
  protected ServiceID servID;
  
  
  //=============== EVENT HANDLING  ==========================
  
  protected List listenerList =  new ArrayList();
  protected BasicRegistrationLandlord eventlandlord; 
  
  protected long seqNum = 0L; // So we can use the sequence no.. 
  

  public EventRegistration register(long eventType,
                                    MarshalledObject clientdata, 
                                    RemoteEventListener listener, 
                                    long duration)
	                                 throws RemoteException {
	// This calls into the landlord to add to the listeners
	// these can then fire these down .. 
	
    EventCookie cookie = new EventCookie(eventType); 
    return eventlandlord.addEventReg(cookie, eventType, 
	                             clientdata, listener,
	                             this,duration);
	
      }

  // Notify all listeners that have registered interest for
  // notification on this event type.  The event instance 
  // is lazily created using the parameters passed into 
  // the fire method.

  protected void fireNotify(RemoteEvent remoteEvent) {
        	
        	
     // First, check the list for registrations whose lease
     // has expired
     // in reverse order (to make us immune from 
     // compaction)
        
     long now = System.currentTimeMillis();
     for (int i=listenerList.size()-1 ; i >= 0 ; i--) {
          Registration reg = (Registration) listenerList.get(i);
          if (reg.expiration < now) {
                reg.cancelled();
                listenerList.remove(i);
          }
     }
        
    long remoteEventType = remoteEvent.getID(); 
        
        
    // Now scan through the list calling out to the notifiers.. 
    for (int i=0, size=listenerList.size() ; i<size ; i++) {
         Registration reg = (Registration) listenerList.get(i);
         EventCookie cookie =  (EventCookie) reg.getCookie();
           
         if (cookie.eventType == remoteEventType) {
             try {
            	   // SEND THE EVENT TO THE LISTENER 
          	   reg.listener.notify(remoteEvent);
             } catch (RemoteException ex) {
                  // Just complain...
                   System.err.println("Error notifying listener:"
                                       + ex.getMessage());
             } catch (UnknownEventException ex) {
            
                   // Cancel the registration...
                   //  by setting its expiration so
                   // that it'll be dropped the next time
                   // through.
                   System.err.println("Unknown event, dropping: "
                                      + ex.getMessage());
                   reg.expiration = 0;
            }
        }
    }
  }
    
// ============= File Store Service  ================
    
  public  FileStoreService() throws RemoteException {	
  	
  	eventlandlord = 
  	              new BasicRegistrationLandlord(listenerList, 
  	                            new LandlordLease.Factory());
  }
   
  
  public RemoteResource getParentFile(RemoteResource dir) 
                                          throws RemoteException{
   	
    File file = (File)dir.getContents();
    File parent = file.getParentFile();
   	
    if (parent != null) {
        RemoteResource parentDir = new RemoteResource();

        parentDir.setContents (parent);
        parentDir.setName(parent.getName());
        parentDir.setDirectory(parent.isDirectory());
   	
         // Fire an event 
        fireNotify(new FileUseEvent(this, servID, 
                                    seqNum++, null));	
        return parentDir;
    } else {
        return null;
    }
  }

  public RemoteResource[] listFiles(RemoteResource dir) 
                                         throws RemoteException{
   	
    File file = (File)dir.getContents();
    File [] filelist = file.listFiles();
    
    if (filelist != null) {
   	
         RemoteResource[] returnlist= 
                            new RemoteResource[filelist.length];
   	
         for (int i = 0; i< filelist.length; i++){
           returnlist[i] = new RemoteResource();
           returnlist[i].setContents (filelist[i]);
           returnlist[i].setName(filelist[i].getName());
           returnlist[i].setDirectory(filelist[i].isDirectory());
         }
         // Fire an event 
         fireNotify(new FileUseEvent(this, servID, 
                                     seqNum++, null));	
         return returnlist;
    } else {
    	 return null;
    }
  }


   
  public void serviceIDNotify(ServiceID id) {
       // This is used by Join Manager 
    servID = id;
    System.out.println("New version The service ID is " + id);
  }
    
  
   // 
  public  RemoteResource getRoot()  {
  	
   File f = new  File( File.separator);
   RemoteResource rootDir = new RemoteResource();
   
   rootDir.setContents (f);
   rootDir.setName(f.getName());
   rootDir.setDirectory(f.isDirectory());	
   	
   return rootDir;
  }

}
