// This proxy creates an adaptor proxy object that maps between the interface
// provided to the Jini framework and the RMI interface used to communicate
// with the remote service

package jiniexamples.chapter8;

import jiniexamples.chapter4.RemoteResource;
import java.io.Serializable;
import java.rmi.MarshalledObject;
import java.rmi.RemoteException;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.EventRegistration;


public class RemoteResourceServiceProxy  
                                implements RemoteStoreWithEvents, 
                                             Serializable {

    // Manage the local state associated with the service..
    
    protected RemoteFileStore backend;
    protected RemoteResource currentDir;
    
    public RemoteResourceServiceProxy() { 
    }
    
    public RemoteResourceServiceProxy(RemoteFileStore backend, 
                                      RemoteResource dir) {
        this.backend = backend;
        currentDir = dir;
    }
    
    // Note each of the methods map between the methods in the 
    // RemoteResourceStoreinterface and the RemoteFileStore  
    // interface. 

    
     // GETCURRENTDIR
    public RemoteResource getCurrentDir(){
            return currentDir;
         
    }
  
     // SETCURRENTDIR
    public void setCurrentDir(RemoteResource dir){
           currentDir = dir;
       
    }
    
    // GETPARENT
    public RemoteResource getParent(RemoteResource dir) 
                                        throws RemoteException {
            return backend.getParentFile(dir);
    }

    // CHANGEUPDIR
    public void changeUpdir() throws RemoteException {
            RemoteResource parent = 
                               backend.getParentFile(currentDir);
            if (parent != null)
                      currentDir =parent;
    }
    
    // LISTRESOURCES
    public RemoteResource[] listResources() 
                                        throws RemoteException {
            return backend.listFiles(currentDir);
    }
    
    // The event registration stuff..
    
    public EventRegistration register( long eventType,
                                   MarshalledObject data, 
                                   RemoteEventListener listener, 
                                   long duration) 
                                         throws RemoteException {
      
            return backend.register(eventType,data,listener,
                                    duration);
    }
}
