package jiniexamples.chapter9;

import jiniexamples.chapter8.BasicRegistrationLandlord;
import jiniexamples.chapter8.Registration;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.rmi.server.UnicastRemoteObject;

import net.jini.core.lookup.ServiceID;
import net.jini.discovery.LookupDiscoveryManager;
import net.jini.lookup.ServiceIDListener;
import net.jini.lookup.JoinManager;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.EventRegistration;
import net.jini.core.event.UnknownEventException;
import com.sun.jini.lease.landlord.LandlordLease;


import java.util.List;
import java.util.HashMap;
import java.util.ArrayList;


// =============== EventListCookie ======================
// Define a new type of cookie to use for accessing
// leases.
class EventListCookie implements Serializable {    
   protected Object source;    
   protected long eventType;        
   
   EventListCookie(Object source, long eventType) {
        this.source = source;
        this.eventType = eventType;    
   }        
   
   public boolean equals(Object o) {
        if (!(o instanceof EventListCookie))
                  return false;                
         
        EventListCookie cookie = (EventListCookie) o;
        return cookie.eventType == eventType && 
                       cookie.source.equals(source);
   }
}
// =============== end of EventListCookie ======================

public class DisplayEventService extends UnicastRemoteObject 
                                 implements RemoteDisplayEv, 
                                            RemoteEventListener, 
                                            ServiceIDListener  {
  
  protected ServiceID servID;
  
  // This is a hash map of all of the Landlords
  // for the individual registration lists; it
  // is keyed by 'source'.
  protected HashMap landlords = new HashMap();

  LandlordLease.Factory factory = new LandlordLease.Factory();
 
  public  DisplayEventService() throws RemoteException {	
  }

  public EventRegistration register(long eventType,
    				    Object source,
                                    RemoteEventListener listener, 
                                    long duration)
	                                  throws RemoteException {
    
     System.out.println("Register  " + eventType);
     BasicRegistrationLandlord eventlandlord = 
                       (BasicRegistrationLandlord) landlords.get(source);
     if (eventlandlord == null) {
	eventlandlord = new BasicRegistrationLandlord(
		                           new ArrayList(), 
		                           new LandlordLease.Factory());
        landlords.put(source,eventlandlord);     	                           
     }
     EventListCookie cookie = new EventListCookie(source, eventType);	
     return eventlandlord.addEventReg(cookie,eventType, null, 
                                     listener,this,duration);
  }

  // Notify all listeners that have registered interest for
  // notification on this event type.  The event instance 
  // is lazily created using the parameters passed into 
  // the fire method.


  public void notify(RemoteEvent remoteEvent) {
        	
        
    Object source = remoteEvent.getSource();
        
    // Find the lanlord managing the list of listeners for
    // this source.
    BasicRegistrationLandlord eventlandlord = (BasicRegistrationLandlord) landlords.get(source);

    if (eventlandlord != null) { 
      // Get the listener list from the landlord
        List listenerList = eventlandlord.getRegs();       
            
        long now = System.currentTimeMillis();
        for (int i=listenerList.size()-1 ; i >= 0 ; i--) {
           Registration reg = (Registration) listenerList.get(i);
           if (reg.getExpiration() < now) {
               reg.cancelled();
               listenerList.remove(i);
           }
        }
        long remoteEventType = remoteEvent.getID(); 
       
        // Now scan through the list calling out to the notifiers.. 
        for (int i=0, size=listenerList.size() ; i<size ; i++) {
           Registration reg = (Registration) listenerList.get(i);
           EventListCookie cookie =  (EventListCookie) reg.getCookie();     
           if (cookie.eventType == remoteEventType) {
           	
                try {
                     // DISPLAY THE EVENT  
                     display(remoteEvent);
                     // FORWARD THE EVENT TO THE LISTENER
              	     reg.getListener().notify(remoteEvent);
                } catch (RemoteException ex) {
                      // Just complain...
                      System.err.println(
                               "Error notifying remote listener: " +
                               ex.getMessage());
                } catch (UnknownEventException ex) {
                     // Cancel the registration...
                     //  by setting its expiration so
                     // that it'll be dropped the next time
                     System.err.println("Unknown event, dropping: " +
                          ex.getMessage());
                         reg.setExpiration(0);
            	}
          }
        }
           
      } else {
       		// IGNORE ... as source not registered..
       		System.err.println("Unknown source dropping event");
      }
  }
   
 
  public void display(RemoteEvent remoteEvent){
      System.out.println ( " Event type = " + remoteEvent.getID() +
    	                   " with sequence number = "+ remoteEvent.getSequenceNumber() );
  }

   // The code added to make this a Jini Service 
  public void serviceIDNotify(ServiceID id) {
      // This is used by Join Manager 
      servID = id;
      System.out.println("Display Event Service Registered The service ID is " + id);
  }
       
  public static void main(String[] args) throws Exception {
      System.setSecurityManager(new RMISecurityManager());
      String[] allgroups = new String[] { "" };
          
      DisplayEventService displayservice = 
                            (DisplayEventService) new DisplayEventService();
      DisplayServiceProxy serviceproxy = 
                            new DisplayServiceProxy(displayservice, displayservice);
      LookupDiscoveryManager discoverymanager = 
                            new LookupDiscoveryManager(allgroups, null, null);
        JoinManager manager = 
                            new JoinManager(serviceproxy, null,displayservice, 
                                            discoverymanager, null);
  }
  
}