// This proxy creates an adaptor proxy object that maps between the interface
// provided to the Jini framework and the RMI interface used to communicate
// with the remote service

package jiniexamples.chapter9;

import java.io.Serializable;
import java.rmi.Remote;
import java.rmi.RemoteException;

import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.EventRegistration;
import net.jini.core.event.UnknownEventException;

public class DisplayServiceProxy  implements DisplayEv, Serializable {
	
    // The local state associated with the service..
    protected RemoteDisplayEv backend;
    protected RemoteEventListener eventInterface;
    
    public DisplayServiceProxy(RemoteDisplayEv backend, 
                               RemoteEventListener eventInterface){
        this.backend = backend;
        this.eventInterface = eventInterface;
    }
    // each of the methods map between the methods in DisplayEv    
    // interface and the RemoteDisplayEv interface.     
    // Each method also catches the expections	
    
    public RemoteEventListener getEventListener(){
    	return eventInterface;
    }
    
    public EventRegistration register(long eventType,
    				      Object source,
                                      RemoteEventListener listener, 
                                      long duration) {
      try {
         return backend.register(eventType,source,listener,duration);
      } catch (RemoteException ex) {
         System.out.println( "Couldn't  register events: " +
                  ex.getMessage());
         return null;   
      }
    }
}
