package jiniexamples.chapter9;

import jiniexamples.chapter8.RemoteStoreWithEvents;
import jiniexamples.chapter8.FileUseHandler;
import jiniexamples.chapter8.FileUseEvent;


import java.util.Hashtable;
import java.util.Enumeration;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;

import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.Box;
import javax.swing.BoxLayout;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.Color;

import net.jini.lookup.ServiceDiscoveryManager;
import net.jini.lookup.LookupCache;
import net.jini.lookup.ServiceDiscoveryListener;
import net.jini.lookup.ServiceDiscoveryEvent;
import net.jini.lookup.entry.Name;
import net.jini.core.lookup.ServiceTemplate;
import net.jini.core.lookup.ServiceItem;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.EventRegistration;
import net.jini.core.lease.Lease;
import net.jini.lease.LeaseRenewalManager;

public class FileActivityMonitor extends JPanel
                            implements ServiceDiscoveryListener{

  protected static String filename;
  protected Box serviceList;
  protected ServiceDiscoveryManager serviceManager;
  protected LookupCache serviceCache;
  protected RemoteEventListener eventListener = null;
  protected DisplayEv evntservice;
  protected static Hashtable knownRegistrations = new Hashtable();
  protected Hashtable actvityIndicators = new Hashtable();

  // === ServiceDiscovery Listener Interface=====
  public void serviceAdded(ServiceDiscoveryEvent ev) {

     EventRegistration er;
     System.out.println("SERVICE ADDED");
     ServiceItem item = ev.getPostEventServiceItem();
     JButton service_label = new JButton(serviceName(item));
     service_label.setBackground(Color.white );
     serviceList.add(service_label);
     serviceList.validate();

     LeaseRenewalManager lrm =
                        serviceManager.getLeaseRenewalManager();
     RemoteStoreWithEvents serviceProxy =
                           (RemoteStoreWithEvents) item.service;
     try {
         if (evntservice ==  null) {
         // No event delegate so register directly .
           er = serviceProxy.register(FileUseEvent.Type,null,
                                    eventListener,Lease.FOREVER);
           lrm.renewUntil(er.getLease(), Lease.FOREVER, null);
           knownRegistrations.put(item.serviceID, er);
         } else {
           er = serviceProxy.register(FileUseEvent.Type,null,
                    evntservice.getEventListener(),Lease.FOREVER);
           lrm.renewUntil(er.getLease(), Lease.FOREVER, null);

           er = evntservice.register(er.getID(),er.getSource(),
                                   eventListener, Lease.FOREVER);
           lrm.renewUntil(er.getLease(), Lease.FOREVER, null);
         }
         actvityIndicators.put(item.serviceID, service_label);

     } catch (RemoteException ex) {
                System.out.println("Error registering for events"
                                    + ex);
     }

   }

   public void serviceRemoved(ServiceDiscoveryEvent ev) {
     System.out.println("SERVICE REMOVED");
     ServiceItem item = ev.getPreEventServiceItem();

     knownRegistrations.remove(item.serviceID);
     Component  activityIndicator =
                (Component)actvityIndicators.get(item.serviceID);

     serviceList.remove(activityIndicator);
     serviceList.validate();
     actvityIndicators.remove(item.serviceID);
   }

   public void serviceChanged(ServiceDiscoveryEvent ev) {
     System.out.println("SERVICE" +
                         ev.getPostEventServiceItem().serviceID +
                         "Changed");
   }
 // == End of ServiceDiscovery Interface ========

   public FileActivityMonitor(ServiceDiscoveryManager sdm )  {

     evntservice = registerwithDisplayEvents(sdm);

      // Now set up the cache...

     Class[] name = new Class[] {RemoteStoreWithEvents.class};
     ServiceTemplate template = 
                            new ServiceTemplate(null,name,null);
     try {
         serviceManager = sdm;
         serviceCache = 
          serviceManager.createLookupCache(template, null, this);
         eventListener = new FileUseHandler(actvityIndicators);
     } catch (RemoteException e) {
             System.out.println ("Remote Errror: " + e);
     }

     // Label
     JLabel toolLabel= new JLabel("Activity on File Stores");
     // Service List
     serviceList = new Box(BoxLayout.Y_AXIS);

     JScrollPane listPane = new JScrollPane(serviceList);

     setLayout(new BorderLayout());
     add(toolLabel, BorderLayout.NORTH);
     add(listPane, BorderLayout.CENTER);
  }

  public DisplayEv registerwithDisplayEvents(
                                   ServiceDiscoveryManager sdm){

     System.out.println ("Look for  Display Event Service ");

     ServiceTemplate eventservicetemplate =
               new ServiceTemplate(null,
                                   new Class[] {DisplayEv.class},
                                   null);
     ServiceItem eventserviceitem = null;

     try {
         //Block until a list of matching services is found
         eventserviceitem = 
                 sdm.lookup(eventservicetemplate, null, 200000);
         // Set the list to display
         if ( eventserviceitem != null) {
             System.out.println ("Display Event Service Found" );
             return (DisplayEv) eventserviceitem.service;
         }
     } catch (RemoteException e) {
          System.out.println ("Search remote  exception: " + e);
     } catch (InterruptedException e) {
          System.out.println ("Search interupt exception:" + e);
     }

     System.out.println ("No Display Event Service" );
     return null;
  }


  protected String serviceName (ServiceItem item) {
     for (int i=0 ; i<item.attributeSets.length ; i++) {
         if (item.attributeSets[i] instanceof Name) {
             return  ((Name)  item.attributeSets[i]).name;
         }
     }
     return "Unamed Service";
  }

  public static void shutdown(){
    // Cancel Leases
     try {
           for (Enumeration el = knownRegistrations.elements() ;
                            el.hasMoreElements() ;) {
              EventRegistration er = 
                            (EventRegistration)el.nextElement();
              er.getLease().cancel();
           }
         } catch (Exception ex)  {
           System.out.println ("Problem cancelling lease:" + ex);
         }

   }


   public static void main (String[] args)
    {
      try {
         System.setSecurityManager (new RMISecurityManager ());

         ServiceDiscoveryManager sdm = 
                          new ServiceDiscoveryManager(null,null);

         FileActivityMonitor contentPane = 
                                   new FileActivityMonitor(sdm);

         JFrame monitorframe = 
                               new JFrame("File Store Monitor");
         monitorframe.addWindowListener(new WindowAdapter() {
                 public void windowClosing(WindowEvent e) {
                             shutdown();;
                             System.exit(0);
                        }
                });

         monitorframe.setContentPane(contentPane);
         monitorframe.setSize(200,300);
         monitorframe.setVisible(true);
       } catch (Exception e)  {
         System.out.println ("main() exception: " + e);
        }
     }
}

